import os
import shutil
import zipfile
import subprocess
from datetime import datetime
import chardet

# 设置每个zip文件内的最大文件数量
max_files_per_zip = 100  # 可以根据需要调整

source_folder = '/www/wwwroot/led/ftp'
temp_folder = '/mnt/msaq/msaq/temp'  # 创建临时文件夹
target_folder = '/mnt/msaq/msaq'

# 已处理文件的记录文件路径
processed_files = '/www/wwwroot/led/processed_files.txt'
processed_files_record = os.path.join(os.getcwd(), processed_files)


def has_permissions(path):
    return os.access(path, os.R_OK | os.W_OK)


def is_file_processed(file_name):
    if not os.path.exists(processed_files_record):
        open(processed_files_record, 'w').close()
    with open(processed_files_record, 'r') as file:
        processed_files = file.read().splitlines()
    return file_name in processed_files


def mark_file_as_processed(file_name):
    with open(processed_files_record, 'a') as file:
        file.write(file_name + '\n')


def detect_and_mount_usb():
    # 查找 USB 设备
    process = subprocess.Popen(
        ['lsblk', '-do', 'name,tran'], stdout=subprocess.PIPE)
    output, _ = process.communicate()
    lines = output.decode().split('\n')
    part = None
    for line in lines:
        if 'usb' in line:
            part = line.split()[0]
            break

    if part:
        mount_point = '/mnt/msaq'
        if not os.path.exists(mount_point):
            os.makedirs(mount_point)

        # 检查设备是否有分区
        process = subprocess.Popen(
            ['lsblk', '/dev/' + part], stdout=subprocess.PIPE)
        output, _ = process.communicate()
        lines = output.decode().split('\n')
        has_partition = any('part' in line for line in lines)

        # 挂载设备
        try:
            if has_partition:
                # 挂载分区（如 /dev/sdb1）
                subprocess.check_call(
                    ['sudo', 'mount', '/dev/' + part + '1', mount_point])
            else:
                # 挂载整个设备（如 /dev/sdb）
                subprocess.check_call(
                    ['sudo', 'mount', '/dev/' + part, mount_point])

            # 检查挂载点权限
            if has_permissions(os.path.join(mount_point, 'msaq')):
                print('USB设备已经挂载成功.')
                return mount_point
            else:
                print('挂载 USB 设备失败：权限不足.')
                subprocess.check_call(['sudo', 'umount', mount_point])  # 卸载设备
        except subprocess.CalledProcessError as e:
            print(f'挂载 USB 设备失败: {e}')
    else:
        print('没有找到 USB 设备。')
    return None


def move_and_zip_files():
    if os.path.exists(source_folder) and has_permissions(source_folder):
        for file_name in os.listdir(source_folder):
            if not is_file_processed(file_name):
                src_file = os.path.join(source_folder, file_name)
                if os.path.isfile(src_file) and os.access(src_file, os.R_OK | os.W_OK):
                    byte_path = os.fsencode(src_file)
                    encoding = chardet.detect(byte_path)['encoding']
                    encoded_file_path = byte_path.decode(encoding)
                    base_name = os.path.basename(encoded_file_path)
                    dst_file = os.path.join(temp_folder, base_name)
                    shutil.move(src_file, dst_file)

                    if len(os.listdir(temp_folder)) >= max_files_per_zip:
                        zip_and_clear_temp_folder()

        # 检查是否有剩余文件未打包
        if os.listdir(temp_folder):
            zip_and_clear_temp_folder()


def zip_and_clear_temp_folder():
    zip_path = os.path.join(target_folder, 'msaq_' +
                            datetime.now().strftime("%Y%m%d_%H%M%S") + '.zip')
    with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
        for file_name in os.listdir(temp_folder):
            file_path = os.path.join(temp_folder, file_name)
            zipf.write(file_path, arcname=file_name)
            os.remove(file_path)
            mark_file_as_processed(file_name)


# 主程序
mount_point = detect_and_mount_usb()
if mount_point is not None:
    if not os.path.exists(temp_folder):
        os.makedirs(temp_folder, exist_ok=True)
        os.chmod(temp_folder, 0o777)
        if not has_permissions(temp_folder):
            print('临时文件夹创建失败.')
    move_and_zip_files()
    print('处理完成，所有文件已打包。')
